var gg = module.exports = require("./gg");
var gui = global.window.nwDispatcher.requireNwGui();

var userStatusCallback = null;
var logicIsReady = false;

module.exports.setup = function(options) {
    var appKey = options.appKey;
    var statusCallback = options.onStatus;
    var comment = {};
    if (options.udid)
        comment.udid = options.udid;

    userStatusCallback = statusCallback;
    return gg.setup_(appKey, JSON.stringify(comment), onStatus);
};

module.exports.startRightspot = function(name, force) {
    return withCookiesSet(function() {
        return gg.startRightspot_(name, force);
    });
};

module.exports.getDevices = function(platform, callback) {
    if (!logicIsReady) {
        return callback({
            wasFound: false,
            reason: "Logic is not ready yet"
        });
    }

    return withCookiesSet(function() {
        return gg.getDevices_(platform, function(x) {
            setTimeout(function() {
                return callback(JSON.parse(x));
            }, 0);
        });
    });
};

module.exports.installProduct = function(platform, token, devices, appId, callback) {
    if (!logicIsReady) {
        return callback({
            wasInstalled: false,
            reason: "Logic is not ready yet"
        });
    }

    if (devices.length === 0) {
        callback({ wasInstalled: true });
        return "ok";
    }

    return withCookiesSet(function() {
        return gg.installProduct_(platform, token, JSON.stringify(devices), appId, function(x) {
            setTimeout(function() {
                return callback(JSON.parse(x));
            }, 0);
        });
    });
};

module.exports.queryForCookies = function(callback) {
    var win = gui.Window.get();
    return win.cookies.getAll({
        domain: "google.com"
    }, function(cookies) {
        var result = {};
        cookies.forEach(function(cookie) {
            if (["SID", "HSID", "APISID", "SSID"].indexOf(cookie.name) !== -1) {
                result[cookie.name] = cookie.value;
            }
        });
        return callback(result);
    });
};

module.exports.setCookies = function(cookies) {
    return gg.setCookies_(JSON.stringify(cookies));
};

module.exports.getInstalledApps = function(platform, callback) {
    if (!logicIsReady) {
        return callback({
            apps: [],
            reason: "Logic is not ready yet"
        });
    }

    return withCookiesSet(function() {
        return gg.getInstalledApps_(platform, function(x) {
            setTimeout(function() {
                return callback(JSON.parse(x));
            }, 0);
        });
    });
};

module.exports.statusString = statusString;

function withCookiesSet(callback) {
    return module.exports.queryForCookies(function(cookies) {
        module.exports.setCookies(cookies);
        return callback();
    });
}

function onStatus(status) {
    setTimeout(function() {
        if (status === gg.STATUS_LOGIC_READY) {
            logicIsReady = true;
            withCookiesSet(function() {
            });
        } else if (status == gg.STATUS_NOT_INITIALIZED) {
            logicIsReady = false;
        }

        if (userStatusCallback) {
            return userStatusCallback(status);
        }
    }, 0);
}

function statusString(status) {
    switch (status) {
    case gg.STATUS_NOT_INITIALIZED: return "Not Initialized";
    case gg.STATUS_NOT_STARTED: return "Not Started";
    case gg.STATUS_WAITING_TO_OPEN: return "Waiting To Open";
    case gg.STATUS_SHOWING: return "Showing";
    case gg.STATUS_INSTALL: return "Install";
    case gg.STATUS_TIMEOUT: return "Timeout";
    case gg.STATUS_CLOSE: return "Close";
    case gg.STATUS_LATER: return "Later";
    case gg.STATUS_NOT_NOW: return "Not Now";
    case gg.STATUS_LOGIC_READY: return "Logic Ready";
    default: return "Unknown";
    }
}
